package;

import flash.utils.Timer;
import openfl.Assets;
import flash.geom.Rectangle;
import flash.net.SharedObject;
import org.flixel.addons.FlxEmitterExt;
import org.flixel.FlxObject;
import org.flixel.FlxButton;
import org.flixel.FlxG;
import org.flixel.FlxPath;
import org.flixel.FlxSave;
import org.flixel.FlxSprite;
import org.flixel.FlxState;
import org.flixel.FlxText;
import org.flixel.FlxTilemap;
import org.flixel.util.FlxMath;

enum GameState {
	Chair;
	Person;
	Car;
	Parasail;
}

class Game extends FlxState
{
	private var stageWidth:Int = 1100;
	private var stageHeight:Int = 650;
	
	private var background:FlxSprite;
	
	private var map:FlxTilemap;
	private var player:FlxSprite;
	private var chairJiggles:Int = 0;
	private var chairJiggleTimer:Timer;
	private var playerIsDead:Bool = false;
	private var previousPlayerVelocityY:Float;
	
	private var crate:FlxSprite;
	private var bomb:FlxSprite;
	private var paraSailBag:FlxSprite;
	private var carKeys:FlxSprite;
	private var car:FlxSprite;
	private var carStartingY:Float = 360;
	private var trashedCar:FlxSprite;
	private var isCarCrahsed:Bool = false;
	private var hasBombExploded:Bool = false;
	
	private var bombOverlay:FlxSprite;
	private var bombOverlayNumbers:FlxSprite;
	private var previousFrame:Int;
	private var explosion:FlxEmitterExt;
	
	private var actionDelay:Timer;
	private var carKeysAquired:Bool = false;
	private var paraSailBagAquired:Bool = false;
	private var state:GameState;
	
	private var victory:Bool = false;
	
	private var test:FlxText;
	private var interact:FlxText;
	
	override public function create():Void
	{
		FlxG.bgColor = 0xff7FB7FF;
		
		super.create();
		
		FlxG.fade(0xff000000, 1, true, null, false);
		
		map = new FlxTilemap();
		map.loadMap(Assets.getText("assets/map.txt"), "assets/tiles.png");
		add(map);
		FlxG.worldBounds.make(map.x, map.y, map.width, map.height);
		
		background = new FlxSprite();
		background.loadGraphic("assets/background.png");
		//background.x = -550;
		//background.y = -325;
		add(background);
		
		crate = new FlxSprite();
		crate.loadGraphic("assets/crate.png");
		crate.x = 410;
		crate.y = 190;
		add(crate);
		
		bomb = new FlxSprite();
		bomb.loadGraphic("assets/bomb.png");
		bomb.x = 390;
		bomb.y = 169;
		bomb.drag.x = 200;
		bomb.acceleration.y = 400;
		add(bomb);
		
		paraSailBag = new FlxSprite();
		paraSailBag.loadGraphic("assets/parasailBag.png");
		paraSailBag.x = 550;
		paraSailBag.y = 169;
		add(paraSailBag);
		
		carKeys = new FlxSprite();
		carKeys.loadGraphic("assets/carKeys.png");
		carKeys.x = 600;
		carKeys.y = 350;
		add(carKeys);
		
		car = new FlxSprite();
		car.loadGraphic("assets/car.png", true, false, 171, 60);
		car.addAnimation("default", [0], 0, false);
		car.addAnimation("driven", [1], 0, false);
		car.play("default");
		car.x = 750;
		car.y = carStartingY;
		car.drag.x = 200;
		car.acceleration.y = 400;
		add(car);
		
		bombOverlay = new FlxSprite();
		bombOverlay.loadGraphic("assets/bombOverlayBackground.png");
		bombOverlay.x = 10;
		bombOverlay.y = 10;
		bombOverlay.scrollFactor.x = 0;
		bombOverlay.scrollFactor.y = 0;
		add(bombOverlay);
		
		bombOverlayNumbers = new FlxSprite();
		bombOverlayNumbers.loadGraphic("assets/bombOverlayNumbers.png", true, false, 130, 120);
		bombOverlayNumbers.x = bombOverlay.x + 104;
		bombOverlayNumbers.y = bombOverlay.y + 39;
		bombOverlayNumbers.scrollFactor.x = 0;
		bombOverlayNumbers.scrollFactor.y = 0;
		bombOverlayNumbers.addAnimation("default", [0,0,0,1,1,1,2,2,2,2,3,3,3,4,4,4,5,5,5,6,6,6,7,7,7,8,8,8,9,9,9,10,10,10,11,11,11], 2, false);
		bombOverlayNumbers.play("default");
		add(bombOverlayNumbers);
		
		actionDelay = new Timer(300, 1);
		chairJiggleTimer = new Timer(500, 1);
		state = GameState.Chair;
		
		test = new FlxText(0, 0, 100, "floored");
		test.scrollFactor.x = 0;
		test.scrollFactor.y = 0;
		//add(test);
		interact = new FlxText(0, 0, 100, "E/SPACE");
		interact.color = 0xff000000;
		add(interact);
		
		player = new FlxSprite();
		player.loadGraphic("assets/chair.png");
		player.x = 330;
		player.y = 200;
		player.drag.x = 200;
		player.acceleration.y = 400;
		add(player);
		FlxG.camera.follow(player);
		
		explosion = new FlxEmitterExt();
		//explosion.setRotation(0, 0);
		explosion.setMotion(0, 100, 1, 360, 500, 3.5);
		explosion.makeParticles("assets/explosionParticle.png", 200, 0, true, 0);
		explosion.setAlpha(1, 1, 0, 0);
		explosion.gravity = 100;
		add(explosion);
		
		// TODO remove this before release
		//escapeChair();
	}

	override public function update():Void
	{
		super.update();
		
		if (!hasBombExploded && bombOverlayNumbers.finished) {
			blowUp();
		}
		
		interact.visible = false;
		
		if (victory) {
			interact.visible = true;
		}
		
		FlxG.collide(bomb, crate);
		FlxG.collide(map, crate);
		FlxG.collide(player, map);
		if (FlxG.collide(bomb, map) && !hasBombExploded) {
			blowUp();
		}
		var carCollidesMap:Bool = FlxG.collide(car, map);
		if (!carCollidesMap && car.y > carStartingY + 10) {
			car.angularVelocity = 15;
		}
		else if (car.angularVelocity > 1 && carCollidesMap) {
			explodeCar();
		}
		
		if (state == GameState.Person) {
			if (!playerIsDead) {

				if (player.isTouching(FlxObject.FLOOR) && previousPlayerVelocityY  > 300 && player.y > 500) {
					killPlayer();
					FlxG.play("FALL");
				}
				
				if (!victory && player.isTouching(FlxObject.FLOOR) && player.y > 700) {
					playerWin();
				}
				
				if (paraSailBagAquired && player.y > 400 && !player.isTouching(FlxObject.FLOOR)&& !actionDelay.running) {
					interact.visible = true;
					if (FlxG.keys.E || FlxG.keys.SPACE) {
						useParasail();
					}
				}
				
				if (FlxG.keys.LEFT || FlxG.keys.A) {
					player.velocity.x = -150;
					player.facing = FlxObject.LEFT;
				}
				if (FlxG.keys.RIGHT || FlxG.keys.D) {
					player.velocity.x = 150;
					player.facing = FlxObject.RIGHT;
				}
				if (player.isTouching(FlxObject.FLOOR) && (FlxG.keys.UP || FlxG.keys.W)) {
					player.velocity.y = -200;
				}
				
				if (!paraSailBagAquired && FlxG.overlap(player, paraSailBag)) {
					interact.visible = true;
					if (FlxG.keys.E || FlxG.keys.SPACE) {
						pickupParasailBag();
					}
				}
				if (!carKeysAquired && FlxG.overlap(player, carKeys)) {
					interact.visible = true;
					if (FlxG.keys.E || FlxG.keys.SPACE) {
						pickupKeys();
					}
				}
				if (FlxG.overlap(player, car) && carKeysAquired && car.angularVelocity < 1) {
					interact.visible = true;
					if ((FlxG.keys.E || FlxG.keys.SPACE) && !actionDelay.running) {
						enterCar();
					}
				}
				
				if (player.isTouching(FlxObject.FLOOR)) {
					if(Math.abs(player.velocity.x ) > 10) {
						if (player.curAnim != "walk") {
							player.play("walk");
						}
					}
					else {
						if (player.curAnim != "idle") {
							player.play("idle");
						}
					}
				}
				else {
					if (player.curAnim != "jump") {
						player.play("jump");
					}
				}
			}
		}
		else if (state == GameState.Chair && !playerIsDead) {
			interact.visible = true;
			if ((FlxG.keys.UP || FlxG.keys.RIGHT || FlxG.keys.LEFT || FlxG.keys.SPACE || FlxG.keys.E  || FlxG.keys.A || FlxG.keys.D || FlxG.keys.W) 
					&& player.isTouching(FlxObject.FLOOR) && bomb.isTouching(FlxObject.FLOOR)) {
				if (chairJiggles > 7) {
					escapeChair();
					FlxG.play("CHAIR2");
				}else {
					if (chairJiggleTimer.running) {
						bomb.velocity.y = -50;
						bomb.velocity.x = -65;
						chairJiggleTimer.reset();
					}
					
					chairJiggleTimer.start();
					player.velocity.y = -50;
					var shake:Int = 40;
					player.velocity.x += (Math.random() * shake) - (shake / 2);
					chairJiggles++;
					FlxG.play("CHAIR");
				}
			}
		}
		else if (state == GameState.Car) {
			if(!isCarCrahsed){
				interact.visible = true;
				if ((FlxG.keys.E || FlxG.keys.SPACE) && !actionDelay.running) {
					exitCar();
				}
				if (FlxG.keys.LEFT || FlxG.keys.A) {
					if(car.velocity.x > 0){
						car.velocity.x += -20;
					}
				}
				if (FlxG.keys.RIGHT || FlxG.keys.D) {
					car.velocity.x += 25;
				}
			}
		}
		else if (state == GameState.Parasail) {
			if (!playerIsDead) {
				
				if (player.isTouching(FlxObject.FLOOR)) {
					ditchParasail();
				}
				
				if (FlxG.keys.LEFT || FlxG.keys.A) {
					player.velocity.x = -150;
					player.facing = FlxObject.LEFT;
				}
				if (FlxG.keys.RIGHT || FlxG.keys.D) {
					player.velocity.x = 150;
					player.facing = FlxObject.RIGHT;
				}
			}
		}
		
		if (state == GameState.Car) {
			interact.x = car.x;
			interact.y = car.y - 20;
		}
		else{
			interact.x = player.x;
			interact.y = player.y - 20;
		}
		
		previousPlayerVelocityY = player.velocity.y;
		
		if (FlxG.keys.ESCAPE) {
			FlxG.fade(0xff000000, 1, false, gotoMenu, false);
		}
		
		if (previousFrame != bombOverlayNumbers.frame) {
			previousFrame = bombOverlayNumbers.frame;
			FlxG.play("TIMER");
		}
	}
	
	private function playerWin():Void {
		victory = true;
		interact.text = "WIN! ESC for Menu!";
	}
	
	private function killPlayer():Void {
		player.visible = false;
		playerIsDead = true;
		FlxG.fade(0xff000000, 3, false, gotoMenu, false);
	}
	
	private function explodeCar():Void {
		
		isCarCrahsed = true;
		
		trashedCar = new FlxSprite();
		trashedCar.loadGraphic("assets/trashedCar.png", true, false, 70, 110);
		trashedCar.addAnimation("default", [0, 1], 2, true);
		trashedCar.play("default");
		trashedCar.x = car.x;
		trashedCar.y = car.y - 30;
		add(trashedCar);
		remove(car);
		
		if(state == GameState.Car){
			FlxG.fade(0xff000000, 3, false, gotoMenu, false);
		}
	}
	
	private function blowUp():Void {
		if ((state == GameState.Person || state == GameState.Chair) && player.y < carStartingY + 10) {
			killPlayer();
		}
		
		if (state == GameState.Car && car.y < carStartingY + 20) {
			explodeCar();
			killPlayer();
		}
		
		bombOverlay.visible = false;
		bombOverlayNumbers.visible = false;
		FlxG.play("BOMB");
		hasBombExploded = true;
		
		explosion.x = bomb.x;
		explosion.y = bomb.y;
		explosion.start(true, 3.5, 0, 2000);
		explosion.update();
	}
	
	private function gotoMenu():Void {
		FlxG.switchState(new MenuState());
	}
	
	private function enterCar():Void {
		actionDelay.reset();
		actionDelay.start();
		//remove(player);
		player.visible = false;
		FlxG.camera.follow(car);
		car.play("driven");
		FlxG.play("CAR");
		
		state = GameState.Car;
	}
	
	private function exitCar():Void {
		actionDelay.reset();
		actionDelay.start();
		//add(player);
		player.visible = true;
		car.play("default");
		
		// Copy the cars speed and position
		player.velocity.x = car.velocity.x;
		player.velocity.y = car.velocity.y;
		player.x = car.x + car.width / 2;
		player.y = car.y - (player.height - car.height) + 1;
		
		FlxG.camera.follow(player);
		
		state = GameState.Person;
	}
	
	private function useParasail():Void {
		remove(player);
		var x:Float = player.x;
		var y:Float = player.y;
		var height:Float = player.height;
		player = new FlxSprite();
		player.loadGraphic("assets/playerParasail.png", false, true);
		player.x = x;
		player.y = y - (player.height - height) + 1;
		player.drag.x = 200;
		player.drag.y = 50;
		player.acceleration.y = 100;
		add(player);
		FlxG.camera.follow(player);
		
		paraSailBagAquired = false;
		
		FlxG.play("PARASAIL");
		
		state = GameState.Parasail;	
	}
	
	private function escapeChair():Void
	{
		toPerson();
	}
	
	private function ditchParasail():Void {
		toPerson();
	}
	
	private function toPerson():Void {
		player.visible = false;
		remove(player);
		var x:Float = player.x;
		var y:Float = player.y;
		var height:Float = player.height;
		player = new FlxSprite();
		player.loadGraphic("assets/player.png", true, true, 30, 68);
		player.addAnimation("idle", [0], 0, false);
		player.addAnimation("jump", [1], 0, false);
		player.addAnimation("walk", [2,3,4], 3, true);
		player.play("idle");
		add(player);
		player.x = x;
		player.y = y - (player.height - height) + 1;
		player.drag.x = 200;
		player.acceleration.y = 400;
		add(player);
		FlxG.camera.follow(player);
		
		state = GameState.Person;
	}
	
	private function pickupParasailBag():Void {
		remove(paraSailBag);
		paraSailBagAquired = true;
		FlxG.play("ZIPPER");
	}
	
	private function pickupKeys():Void {
		remove(carKeys);
		carKeysAquired = true;
		FlxG.play("KEY");
	}
	
	override public function destroy():Void
	{
		super.destroy();
	}
}